package main

import (
	"context"
	"fmt"
	"github.com/apache/thrift/lib/go/thrift"
	"net"
	"os"
)

func main() {
	host := "<YourPublicThriftIP>"
	port := "9090"

	protocolFactory := thrift.NewTBinaryProtocolFactoryDefault()
	socket, err := thrift.NewTSocket(net.JoinHostPort(host, port)) // 创建连接, 短连接
	if err != nil {
		fmt.Println("failed to create new socket err: " + err.Error())
		os.Exit(1)
	}
	transport := thrift.NewTFramedTransport(socket) // 包装一个TFramedTransport

	if err != nil {
		fmt.Fprintln(os.Stderr, "error resolving address:", err)
		os.Exit(1)
	}
	// 该client关联一个连接，NonThreadSafe
	client := NewTHBaseServiceClientFactory(transport, protocolFactory) // hbase客户端与连接关联
	// 执行连接操作
	if err := transport.Open(); err != nil {
		fmt.Fprintln(os.Stderr, "Error opening socket to "+host+":"+port, " ", err)
		os.Exit(1)
	}
	defer transport.Close() // 结束的时候关闭连接

	ctx := context.Background()

	tableName := "demo_test_table"
	columnFamily := []byte("cf")
	// 数据过期时间，例如设置为7天
	var ttl int32 = 7 * 86400
	columnFamilies := []*TColumnFamilyDescriptor{
		{
			Name:       columnFamily,
			TimeToLive: &ttl,
			// FAST_DIFF
			DataBlockEncoding: TDataBlockEncodingPtr(TDataBlockEncoding_FAST_DIFF),
			// lz4压缩
			CompressionType: TCompressionAlgorithmPtr(TCompressionAlgorithm_LZ4),
		},
	}
	descriptor := &TTableDescriptor{
		TableName: &TTableName{
			Qualifier: []byte(tableName),
		},
		Columns: columnFamilies,
	}
	exists, err := client.TableExists(ctx, descriptor.TableName)
	if err != nil {
		fmt.Println("table exist err :" + err.Error())
		return
	}
	if !exists {
		err = client.CreateTable(ctx, descriptor, nil)
		if err != nil {
			fmt.Println("create table err :" + err.Error())
			return
		}
	}

	put := &TPut{
		Row: []byte("foo-row"),
		ColumnValues: []*TColumnValue{
			{
				Family:    columnFamily,
				Qualifier: []byte("sample"),
				Value:     []byte("bar"),
			},
		},
	}
	err = client.Put(ctx, []byte(tableName), put)
	if err != nil {
		fmt.Println("create table err :" + err.Error())
		return
	}

	get := &TGet{
		Row: []byte("foo-row"),
	}
	result, err := client.Get(ctx, []byte(tableName), get)
	if err != nil {
		fmt.Println("get err :" + err.Error())
		return
	}
	if result != nil {
		fmt.Println("get ret success")
		fmt.Println("Rowkey:" + string(result.GetRow()))
		for _, cv := range result.ColumnValues {
			fmt.Println("Family:", string(cv.Family), "Qualifer:", string(cv.Qualifier), "Value:", string(cv.Value))
		}
	} else {
		fmt.Println("get ret is nil")
	}

}
